# Python 3 server example
from http.server import BaseHTTPRequestHandler, HTTPServer
import time #?
import os #get path
import shutil#copy bytecode
import bpy #do rendering
import urllib #parsing url parameter

hostName = "localhost"
serverPort = 8080
C=bpy.context

class MyServer(BaseHTTPRequestHandler):
    def __init__(self, *args, directory=None, **kwargs):
        self.userDir=os.getcwd()#get current dir
        self.renderFileQualifiedName=self.userDir+"/render.jpg"
        
        super().__init__(*args, **kwargs)

    def do_GET(self):
        #explain the browser that an image is retured
        self.send_response(200)
        self.send_header("Content-type", "image/jpeg")
        self.end_headers()        
       
        self.clearImports()#clean up from last renderjob
      
      
        objFileName=self.getParam("obj")                
        if len(objFileName)>0:        
            print("objFileName",objFileName)
            self.importObj(objFileName)                
        
        
        cameraName=self.getParam("cam")
        rotation=self.getParam("rot")
        
        if len(cameraName)>0 or len(rotation)>0:
            self.setupScene(cameraName,rotation)
        
        
        self.render()        
        self.writeOutput()
        
    
    def getParam(self,parameterName):
        query=urllib.parse.urlparse(self.path).query
        if len(query)>0:            
            queries=urllib.parse.parse_qs(query)
            if parameterName in queries:
                objFileName=queries[parameterName][0] # get the first entry of obj param
                return objFileName
        return ""
        
    def writeOutput(self):
        file=open(self.renderFileQualifiedName,"rb") #open for reading in binary mode               
        shutil.copyfileobj(file, self.wfile)
    
    def clearImports(self):
        counter=0
        collImports=bpy.data.collections["imports"]    
        for o in collImports.objects:           
            bpy.data.objects.remove(o);#materials now have user=0
            
    
        
    
    def setupScene(self,aCameraName,aRotation):
        if (aCameraName in bpy.data.objects):
            camera=bpy.data.objects[aCameraName]
            C.scene.camera=camera
        try:
            rotInt=int(aRotation)
            C.scene.frame_current=rotInt
        except:
            pass #do nothing
        
    
    def importObj(self,objName):
        
        #set imports-layer as active layer
        importsLayer=bpy.context.view_layer.layer_collection.children["imports"]        
        bpy.context.view_layer.active_layer_collection=importsLayer #only worls with layer collections
        
        print("import from currDir",objName)
        try:
            res=bpy.ops.import_scene.obj(filepath=self.userDir+"/obj/"+objName,
                                 use_image_search=False,
                                 filter_glob="*.obj")        
        except:
            print("could not find",objName)
            
    def render(self):
        bpy.ops.render.render()
        bpy.data.images["Render Result"].save_render(self.renderFileQualifiedName);
        
        
        

if __name__ == "__main__":        
    webServer = HTTPServer((hostName, serverPort), MyServer)
    print("Server started http://%s:%s" % (hostName, serverPort))

    try:
        webServer.serve_forever()
    except KeyboardInterrupt:#stop the server using Strg+C on Blender's console
        pass

    webServer.server_close()
    print("Server stopped.")